# -*- coding: utf-8 -*-

from plone.app.testing import PloneSandboxLayer
from plone.app.testing import PLONE_FIXTURE
from plone.app.testing import IntegrationTesting
from plone.app.testing import FunctionalTesting


class Fixture(PloneSandboxLayer):

    defaultBases = (PLONE_FIXTURE,)

    def setUpZope(self, app, configurationContext):
        # Load ZCML
        import collective.upload
        self.loadZCML(package=collective.upload)

    def setUpPloneSite(self, portal):
        # Install into Plone site using portal_setup
        self.applyProfile(portal, 'collective.upload:default')


FIXTURE = Fixture()
INTEGRATION_TESTING = IntegrationTesting(
    bases=(FIXTURE,),
    name='collective.upload:Integration',
    )
FUNCTIONAL_TESTING = FunctionalTesting(
    bases=(FIXTURE,),
    name='collective.upload:Functional',
    )



# -*- coding: utf-8 -*-

PROJECTNAME = 'collective.upload'

# XXX: we don't want to put limits here; that should be user configurable
UPLOAD_EXTENSIONS = u'gif, jpeg, jpg, png'
MAX_FILE_SIZE = 10 * 1024 * 1024  # 10MB; that should be big enough
RESIZE_MAX_WIDTH = 3872  # 10 MP landscape
RESIZE_MAX_HEIGHT = 3872  # 10 MP portrait



# -*- coding: utf-8 -*-

import json

from Acquisition import aq_inner

from five import grok
from zope.container.interfaces import INameChooser
from zope.component import queryMultiAdapter
from zope.interface import Interface
from zope.i18n import translate
from zope.component import getUtility
from zope.app.component.hooks import getSite

from collective.upload.interfaces import IUploadBrowserLayer, IUploadSettings
from collective.upload.behaviors import  IMultipleUpload

from plone.app.content.browser.foldercontents import FolderContentsView
from plone.registry.interfaces import IRegistry

from collective.upload import _

IMAGE_MIMETYPES = ['image/jpeg', 'image/gif', 'image/png']

grok.templatedir('templates')


# TODO: implement drag&drop here
class Folder_Contents(grok.View, FolderContentsView):
    grok.context(IMultipleUpload)
    grok.layer(IUploadBrowserLayer)
    grok.require('cmf.ModifyPortalContent')


# TODO: convert into a folder action: Upload files and images
class Media_Uploader(grok.View):
    """ Handler for the upload process, creation of files, can set a title or
        description, the place to touch if you need extra data saved.
    """
    grok.context(IMultipleUpload)
    grok.require('cmf.AddPortalContent')

    files = []

    def __call__(self, *args, **kwargs):
        if hasattr(self.request, "REQUEST_METHOD"):
            json_view = queryMultiAdapter((self.context, self.request),
                                          name=u"api")
            # TODO: we should check errors in the creation process, and
            # broadcast those to the error template in JS
            if self.request["REQUEST_METHOD"] == "POST":
                if getattr(self.request, "files[]", None) is not None:
                    files = self.request['files[]']
                    title = self.request['title[]']
                    description = self.request['description[]']
                    uploaded = self.upload([files], [title], [description])
                    if uploaded and json_view:
                        upped = []
                        for item in uploaded:
                            upped.append(json_view.getContextInfo(item))
                        return json_view.dumps(upped)
                return json_view()
        return super(Media_Uploader, self).__call__(*args, **kwargs)

    def upload(self, files, title='', description=''):
        loaded = []
        namechooser = INameChooser(self.context)
        if not isinstance(files, list):
            files = [files]
        portal = getSite()
        for item in files:
            if item.filename:
                content_type = item.headers.get('Content-Type')
                id_name = ''
                if title:
                    id_name = namechooser.chooseName(title[0], portal)
                else:
                    id_name = namechooser.chooseName(item.filename, portal)
                portal_type = 'File'
                if content_type in IMAGE_MIMETYPES:
                    portal_type = 'Image'
                name_index = 0
                while name_index < 100:
                    try:
                        self.context.invokeFactory(portal_type, id=id_name, file=item,
                            description=description[0])
                        self.context[id_name].reindexObject()
                        newfile = self.context[id_name]
                        loaded.append(newfile)
                        name_index = 100
                    except:
                        pass
                    name_index = name_index + 1
                    id_name = id_name + '-' + str(name_index)
            if loaded:
                return loaded
            return False


class JSON_View(grok.View):
    """ JSON converter; the jQuery plugin requires this kind of response,
    represent the metadata info of the file.
    """
    grok.context(Interface)
    grok.name('api')
    grok.require('cmf.AddPortalContent')

    json_var = {'name': 'File-Name.jpg',
                'title': '',
                'description': '',
                'size': 999999,
                'url': '\/\/nohost.org',
                'thumbnail_url': '//nohost.org',
                'delete_url': '//nohost.org',
                'delete_type': 'DELETE',
                }

    def __call__(self):
        self.response.setHeader('Content-Type', 'text/plain')
        return super(JSON_View, self).__call__()

    def dumps(self, json_var=None):
        """ """
        if json_var is None:
            json_var = {}
        return json.dumps(json_var)

    def getContextInfo(self, context=None):
        if context is None:
            context = self.context
        context = aq_inner(context)

        info = ''
        if hasattr(context, 'size'):
            context_state = queryMultiAdapter((context, self.request),
                                            name=u'plone_context_state')
            context_name = context_state.object_title()
            context_url = context_state.object_url()

            del_url = context_url
            # TODO: we should check errors in the delete process, and
            # broadcast those to the error template in JS
            info = {'name': context_name,
                    'title': context_name,
                    'description': context.Description(),
                    'url': context_url,
                    'size': context.size(),
                    'delete_url': del_url,
                    'delete_type': 'DELETE',
                    }
            if context.Type() == 'Image':
                info['thumbnail_url'] = context_url + '/image_thumb'
        return info

    def getContainerInfo(self):
        contents = []
        for item in self.context.objectIds():
            item_info = self.getContextInfo(self.context[item])
            if item_info:
                contents.append(item_info)
        return contents

    def render(self):
        return self.dumps(self.getContainerInfo())


messages = {
    'DELETE_MSG': _(u'delete', default=u'Delete'),
    'START_MSG': _(u'start', default=u'Start'),
    }

messageTemplate = "jupload={};jupload.messages = {\n%s};\njupload.config = %s;\n"


class JSVariables(grok.View):
    """ This method generates global JavaScript variables, for i18n and plugin
    configuration.
    """
    grok.context(Interface)
    grok.name('jsvariables')

    def render(self):
        response = self.request.response
        response.setHeader('content-type', 'text/javascript;;charset=utf-8')

        template = ''

        for key in messages:
            msg = translate(messages[key], context=self.request).replace("'", "\\'")
            template = "%s%s: '%s',\n" % (template, key, msg)

        # note trimming of last comma
        return messageTemplate % (template[:-2], self.registry_config())

    def registry_config(self):
        config = {}
        registry = getUtility(IRegistry)
        settings = registry.forInterface(IUploadSettings)
        config['extensions'] = str(settings.upload_extensions.replace(' ', '').replace(',', '|'))
        config['max_file_size'] = settings.max_file_size
        config['resize_max_width'] = settings.resize_max_width
        config['resize_max_height'] = settings.resize_max_height

        return str(config)



# -*- coding: utf-8 -*-

from zope import schema
from zope.interface import Interface

from plone.theme.interfaces import IDefaultPloneLayer

from collective.upload import _
from collective.upload import config


class IUploadBrowserLayer(IDefaultPloneLayer):
    """A marker interface for the theme layer.
    """


class IUploadSettings(Interface):
    """Global upload settings. This describes records stored in the
    configuration registry and obtainable via plone.registry.
    """

    upload_extensions = schema.TextLine(
        title=_(u"Allowed Extensions"),
        description=_(u"allowed_extensions_key",
        default=u"List the file extensions allowed for upload, "
                u"without dot, comma separated"),
        required=False,
        default=config.UPLOAD_EXTENSIONS,
        )

    max_file_size = schema.Int(
        title=_(u"Max file size"),
        description=_(u"max_file_size_key",
                      default=u"The maximum allowed file size in bytes. "
                              u"Please note that intermediate web servers "
                              u"may put limits on this value; ask your "
                              u"Systems Administrator in case of doubt."),
        required=False,
        default=config.MAX_FILE_SIZE,)

    resize_max_width = schema.Int(
        title=_(u"maximum width"),
        description=_(u"resize_max_width_key",
                      default=u"The maximum width for uploaded images."),
        required=False,
        default=config.RESIZE_MAX_WIDTH,
        )

    resize_max_height = schema.Int(
        title=_(u"maximum height"),
        description=_(u"resize_max_height_key",
                      default=u"The maximum height for uploaded images."),
        required=False,
        default=config.RESIZE_MAX_HEIGHT,
        )



# -*- coding: utf-8 -*-

import unittest2 as unittest

from zope.component import getMultiAdapter, queryMultiAdapter
from zope.interface import directlyProvides

from plone.app.testing import TEST_USER_ID
from plone.app.testing import setRoles

from collective.upload.interfaces import IUploadBrowserLayer
from collective.upload.testing import INTEGRATION_TESTING


class JSVariablesViewTestCase(unittest.TestCase):

    layer = INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.request = self.layer['request']
        directlyProvides(self.request, IUploadBrowserLayer)

        setRoles(self.portal, TEST_USER_ID, ['Manager'])
        self.portal.invokeFactory('Folder', 'test-folder')
        setRoles(self.portal, TEST_USER_ID, ['Member'])
        self.folder = self.portal['test-folder']

    def test_jsvariables_view_is_present(self):
        view = queryMultiAdapter((self.folder, self.request), name='jsvariables')
        self.assertTrue(view is not None)

    def test_registry_config(self):
        view = getMultiAdapter((self.folder, self.request), name='jsvariables')

        registry_config = view.registry_config()
        expected = "{'max_file_size': 10485760, 'resize_max_width': 3872, " \
                   "'extensions': 'gif|jpeg|jpg|png', 'resize_max_height': 3872}"
        self.assertEqual(registry_config, expected)


class APITestCase(unittest.TestCase):

    layer = INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.request = self.layer['request']
        directlyProvides(self.request, IUploadBrowserLayer)

        setRoles(self.portal, TEST_USER_ID, ['Manager'])
        self.portal.invokeFactory('Folder', 'test-folder')
        setRoles(self.portal, TEST_USER_ID, ['Member'])
        self.folder = self.portal['test-folder']

    def test_api_view_is_present(self):
        view = queryMultiAdapter((self.folder, self.request), name='api')
        self.assertTrue(view is not None)

    def test_dumps(self):
        view = getMultiAdapter((self.folder, self.request), name='api')

        self.assertEqual(view.dumps(), '{}')
        # TODO: test with content added

    def test_getContextInfo(self):
        #view = getMultiAdapter((self.folder, self.request), name='api')
        # TODO
        NotImplemented

    def test_getContainerInfo(self):
        #view = getMultiAdapter((self.folder, self.request), name='api')
        # TODO
        NotImplemented

    def test_render(self):
        view = getMultiAdapter((self.folder, self.request), name='api')

        self.assertEqual(view.render(), '[]')
        # TODO: test with content added



# -*- coding: utf-8 -*-

from plone.app.registry.browser import controlpanel

from collective.upload import _
from collective.upload.interfaces import IUploadSettings


class UploadSettingsEditForm(controlpanel.RegistryEditForm):

    schema = IUploadSettings
    label = _(u"Upload settings")
    description = _(u"Here you can modify the settings for collective.upload")

    def updateFields(self):
        super(UploadSettingsEditForm, self).updateFields()

    def updateWidgets(self):
        super(UploadSettingsEditForm, self).updateWidgets()


class UploadSettingsControlPanel(controlpanel.ControlPanelFormWrapper):
    form = UploadSettingsEditForm



# -*- coding: utf-8 -*-

from setuptools import setup, find_packages
import os

version = '1.0b2'
long_description = open("README.txt").read() + "\n" + \
                   open(os.path.join("docs", "INSTALL.txt")).read() + "\n" + \
                   open(os.path.join("docs", "CREDITS.txt")).read() + "\n" + \
                   open(os.path.join("docs", "HISTORY.txt")).read()

setup(name='collective.upload',
      version=version,
      description="File upload widget with multiple file selection, "
                  "drag&drop support, progress bars, client-side image "
                  "resizing and preview images.",
      long_description=long_description,
      classifiers=[
        "Development Status :: 4 - Beta",
        "Environment :: Web Environment",
        "Framework :: Plone",
        "Framework :: Plone :: 4.1",
        "Framework :: Plone :: 4.2",
        "Intended Audience :: End Users/Desktop",
        "Intended Audience :: System Administrators",
        "License :: OSI Approved :: GNU General Public License (GPL)",
        "Operating System :: OS Independent",
        "Programming Language :: JavaScript",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2.6",
        "Topic :: Software Development :: Libraries :: Python Modules",
        ],
      keywords='plone jquery upload',
      author='Silvestre Huens',
      author_email='s.huens@gmail.com',
      url='https://github.com/collective/collective.upload',
      license='GPL',
      packages=find_packages('src'),
      package_dir={'': 'src'},
      namespace_packages=['collective'],
      include_package_data=True,
      zip_safe=False,
      install_requires=[
        'setuptools',
        'collective.js.jqueryui',
        'five.grok>=1.2.0',
        'plone.behavior',
        'z3c.jbot',
        ],
      extras_require={
        'test': ['plone.app.testing'],
        },
      entry_points="""
      [z3c.autoinclude.plugin]
      target = plone
      """,
      )



# -*- coding: utf-8 -*-

import unittest2 as unittest

from zope.component import queryMultiAdapter
from zope.interface import alsoProvides
from zope.viewlet.interfaces import IViewletManager

from plone.app.testing import TEST_USER_ID
from plone.app.testing import setRoles

from Products.Five.browser import BrowserView as View

from collective.upload.behaviors import IMultipleUpload
from collective.upload.testing import INTEGRATION_TESTING


class ViewletTest(unittest.TestCase):

    layer = INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.request = self.layer['request']

        setRoles(self.portal, TEST_USER_ID, ['Manager'])
        self.portal.invokeFactory('Folder', 'test-folder')
        setRoles(self.portal, TEST_USER_ID, ['Member'])
        self.folder = self.portal['test-folder']
        # HACK: our Folder object "has" the behavior just for testing purposes
        alsoProvides(self.folder, IMultipleUpload)

    def test_viewlet_is_present(self):
        view = View(self.folder, self.request)
        manager = queryMultiAdapter((self.folder, self.request, view),
                                    IViewletManager, 'plone.portalfooter')
        self.assertTrue(manager is not None)

        manager.update()
        viewlet = [v for v in manager.viewlets if v.__name__ == 'upload.init']
        self.assertEqual(len(viewlet), 1)



# -*- coding: utf-8 -*-

import unittest2 as unittest

from zope.component import getMultiAdapter
from zope.component import getUtility

from plone.app.testing import TEST_USER_ID
from plone.app.testing import logout
from plone.app.testing import setRoles
from plone.registry.interfaces import IRegistry

from collective.upload import config
from collective.upload.config import PROJECTNAME
from collective.upload.interfaces import IUploadSettings
from collective.upload.testing import INTEGRATION_TESTING


class ControlPanelTestCase(unittest.TestCase):

    layer = INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.controlpanel = self.portal['portal_controlpanel']
        setRoles(self.portal, TEST_USER_ID, ['Manager'])

    def test_controlpanel_has_view(self):
        view = getMultiAdapter((self.portal, self.portal.REQUEST),
                               name='upload-settings')
        view = view.__of__(self.portal)
        self.assertTrue(view())

    def test_controlpanel_view_is_protected(self):
        from AccessControl import Unauthorized
        logout()
        self.assertRaises(Unauthorized,
                          self.portal.restrictedTraverse,
                          '@@upload-settings')

    def test_controlpanel_installed(self):
        actions = [a.getAction(self)['id']
                   for a in self.controlpanel.listActions()]
        self.assertTrue('upload' in actions,
                        'control panel was not installed')

    def test_controlpanel_removed_on_uninstall(self):
        qi = self.portal['portal_quickinstaller']
        qi.uninstallProducts(products=[PROJECTNAME])
        actions = [a.getAction(self)['id']
                   for a in self.controlpanel.listActions()]
        self.assertTrue('upload' not in actions,
                        'control panel was not removed')


class RegistryTestCase(unittest.TestCase):

    layer = INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.registry = getUtility(IRegistry)
        self.settings = self.registry.forInterface(IUploadSettings)
        setRoles(self.portal, TEST_USER_ID, ['Manager'])

    def test_upload_extensions_record_in_registry(self):
        self.assertTrue(hasattr(self.settings, 'upload_extensions'))
        self.assertEqual(self.settings.upload_extensions,
                         config.UPLOAD_EXTENSIONS)

    def test_max_file_size_record_in_registry(self):
        self.assertTrue(hasattr(self.settings, 'max_file_size'))
        self.assertEqual(self.settings.max_file_size,
                         config.MAX_FILE_SIZE)

    def test_resize_max_width_in_registry(self):
        self.assertTrue(hasattr(self.settings, 'resize_max_width'))
        self.assertEqual(self.settings.resize_max_width,
                         config.RESIZE_MAX_WIDTH)

    def test_resize_max_height_record_in_registry(self):
        self.assertTrue(hasattr(self.settings, 'resize_max_height'))
        self.assertEqual(self.settings.resize_max_height,
                         config.RESIZE_MAX_HEIGHT)

    def get_record(self, record):
        """ Helper function; it raises KeyError if the record is not in the
        registry.
        """
        prefix = 'collective.upload.controlpanel.IuploadSettings.'
        return self.registry[prefix + record]

    def test_records_removed_on_uninstall(self):
        # XXX: I haven't found a better way to test this; anyone?
        qi = self.portal['portal_quickinstaller']
        qi.uninstallProducts(products=[PROJECTNAME])
        self.assertRaises(KeyError, self.get_record, 'upload_extensions')
        self.assertRaises(KeyError, self.get_record, 'max_file_size')
        self.assertRaises(KeyError, self.get_record, 'resize_max_width')
        self.assertRaises(KeyError, self.get_record, 'resize_max_height')



# -*- coding: utf-8 -*-

from five import grok

from plone.app.layout.viewlets.interfaces import IPortalFooter

from collective.upload.behaviors import IMultipleUpload

grok.templatedir('viewlets')


class MediaUploaderInit(grok.Viewlet):
    """ The viewlet doing the initialization, imports of JavaScript,
    no-conflict jQuery variable assignation, etc.
    """
    grok.context(IMultipleUpload)
    grok.name('upload.init')
    grok.require('cmf.AddPortalContent')
    grok.template('media_uploader_init')
    grok.viewletmanager(IPortalFooter)



# -*- coding: utf-8 -*-

import unittest2 as unittest

from plone.app.testing import TEST_USER_ID
from plone.app.testing import setRoles

from plone.browserlayer.utils import registered_layers

from collective.upload.config import PROJECTNAME
from collective.upload.testing import INTEGRATION_TESTING


class InstallTest(unittest.TestCase):

    layer = INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']

    def test_installed(self):
        qi = getattr(self.portal, 'portal_quickinstaller')
        self.assertTrue(qi.isProductInstalled(PROJECTNAME))

    def test_browserlayer_installed(self):
        layers = [l.getName() for l in registered_layers()]
        self.assertTrue('IUploadBrowserLayer' in layers)


class UninstallTest(unittest.TestCase):

    layer = INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        setRoles(self.portal, TEST_USER_ID, ['Manager'])
        self.qi = self.portal['portal_quickinstaller']
        self.qi.uninstallProducts(products=[PROJECTNAME])

    def test_uninstalled(self):
        self.assertFalse(self.qi.isProductInstalled(PROJECTNAME))

    def test_browserlayer_uninstalled(self):
        layers = [l.getName() for l in registered_layers()]
        self.assertFalse('IUploadBrowserLayer' in layers)
